package supermarketcheckoutsystem;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

public class Supermarket {

	public Map<String,Item> inventory;
	public ArrayList<Customer> customers;
	public Map<String,DiscountPlan> discounts;
	private CashRegister cashRegister;
	
	public Supermarket(CashRegister cashRegister) throws Exception {
		this.inventory = new HashMap<>();
		this.customers = new ArrayList<Customer>();
		
		// Basic discount plans
		DiscountPlan Normal = new DiscountPlan("normal", 0, 0, 0);
		DiscountPlan Prime = new DiscountPlan("prime", 0.2, 0.5, 50);
		DiscountPlan Platinum = new DiscountPlan("platinum", 0.3, 1, 200);
		this.discounts = new HashMap<>();
		this.discounts.put("normal",Normal);
		this.discounts.put("prime",Prime);
		this.discounts.put("platinum",Platinum);
		
		this.cashRegister = cashRegister;
	}

	public void addToInventory(Item item) {
		this.inventory.put(item.name, item);
	}

	public void setPricingPolicy(Item item, double policy) throws Exception{
		if(!this.inventory.containsValue(item)) {
			throw new Exception("Item not in supermarket.");
		}
		item.setPricingPolicy(policy);
	}
	
	public void setPricingPolicyByCategory(Category category, double policy) throws Exception{
		for(Item item : this.inventory.values()) {
			if(item.category == category) {
				setPricingPolicy(item, policy);
			}
		}
	}
	
	public void addCustomer(Customer customer) {
		this.customers.add(customer);
	}

	public void addDiscount(String name, double discount, double deliveryDiscount, double price) throws Exception {
		this.discounts.put(name, new DiscountPlan(name, discount, deliveryDiscount, price));
	}
	
	public void addItemToCustomer(Customer customer, Item item, double amount) throws Exception{
		if(!customers.contains(customer)){
			throw new Exception("Customer not in the supermarket.");
		}
		if(!inventory.containsKey(item.name)) {
			throw new Exception("Item not in inventory.");
		}
		customer.addItemtoPurchase(item, amount);
	}
	
	public void customerCheckout(Customer customer, boolean receipt, boolean delivery) throws Exception {
		if(!customers.contains(customer)){
			throw new Exception("Customer not in the supermarket.");
		}
		cashRegister.customerCheckout(customer, receipt, delivery);
	}
}
